@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2008 GraphicsMagick Group
+% Copyright (C) 2003 - 2015 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -150,7 +150,8 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
 {
   typedef struct _FITSInfo
   {
-    char extensions_exist;
+    char
+      extensions_exist;
     int
       simple,
       bits_per_pixel,
@@ -304,7 +305,15 @@ ReadExtension:
           fits_info.extensions_exist = (*value== 'T') || (*value=='t');
         if (LocaleCompare(keyword,"BITPIX") == 0)
         {
-          fits_info.bits_per_pixel=atoi(value);
+          fits_info.bits_per_pixel=MagickAtoI(value);
+          /*
+            BITPIX valid values:
+              8 -- Character or unsigned binary integer
+             16 -- 16-bit two's complement binary integer
+             32 -- 32-bit two's complement binary integer
+            -32 -- 32-bit floating point, single precision
+            -64 -- 64-bit floating point, double precision
+          */
 	  if(fits_info.bits_per_pixel>0)
             import_options.sample_type = UnsignedQuantumSampleType;
 	  if(fits_info.bits_per_pixel<0)          
@@ -315,13 +324,13 @@ ReadExtension:
           if (keyword[5] == 0) ax_number=-1;          
           else
 	  {
-            if(isdigit(keyword[5]))
-              ax_number = atoi(keyword+5);
+            if(isdigit((int) keyword[5]))
+              ax_number = MagickAtoI(keyword+5);
             else ax_number=-2;			/*unsupported fits keyword*/
           }
 	  y=0;
           if(ax_number>=-1)
-	    y = atoi(value);
+	    y = MagickAtoI(value);
           switch(ax_number)
           {
             case -1:fits_info.number_axes = y; break;
@@ -339,13 +348,13 @@ ReadExtension:
         }
 
         if (LocaleCompare(keyword,"DATAMAX") == 0)
-          fits_info.max_data=atof(value);
+          fits_info.max_data=MagickAtoF(value);
         if (LocaleCompare(keyword,"DATAMIN") == 0)
-          fits_info.min_data=atof(value);
+          fits_info.min_data=MagickAtoF(value);
         if (LocaleCompare(keyword,"BZERO") == 0)
-          fits_info.zero=atof(value);
+          fits_info.zero=MagickAtoF(value);
         if (LocaleCompare(keyword,"BSCALE") == 0)
-          fits_info.scale=atof(value);
+          fits_info.scale=MagickAtoF(value);
         if (LocaleCompare(keyword,"XENDIAN") == 0)
         {
 	  if (LocaleCompare(keyword,"BIG") == 0)
@@ -368,9 +377,48 @@ ReadExtension:
   /*
     Verify that required image information is defined.
   */  
-  if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),
-          "HDU read finished at %Xh, number of pixel is: %d (%d*%d*%d)", (unsigned)TellBlob(image), (unsigned)number_pixels,
-          (unsigned)fits_info.columns, (unsigned)fits_info.rows, (unsigned)fits_info.number_scenes);
+  if (logging)
+    {
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                            "HDU read finished at %Xh, number of pixel is: %d (%d*%d*%d)",
+                            (unsigned) TellBlob(image),
+                            (unsigned) number_pixels,
+                            (unsigned) fits_info.columns,
+                            (unsigned) fits_info.rows,
+                            (unsigned) fits_info.number_scenes);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                            "FITS header:\n"
+                            "    extensions_exist : 0x%02x\n"
+                            "    simple           : %d\n"
+                            "    bits_per_pixel   : %d\n"
+                            "    columns          : %d\n"
+                            "    rows             : %d\n"
+                            "    number_axes      : %d\n"
+                            "    number_scenes    : %d\n"
+                            "    min_data         : %g\n"
+                            "    max_data         : %g\n"
+                            "    zero             : %g\n"
+                            "    scale            : %g",
+                            (int) fits_info.extensions_exist,
+                            fits_info.simple,
+                            fits_info.bits_per_pixel,
+                            fits_info.columns,
+                            fits_info.rows,
+                            fits_info.number_axes,
+                            fits_info.number_scenes,
+                            fits_info.min_data,
+                            fits_info.max_data,
+                            fits_info.zero,
+                            fits_info.scale);
+    }
+
+  if ((fits_info.bits_per_pixel != 8) &&
+      (fits_info.bits_per_pixel != 16) &&
+      (fits_info.bits_per_pixel != 32) &&
+      (fits_info.bits_per_pixel != 64) &&
+      (fits_info.bits_per_pixel != -32) &&
+      (fits_info.bits_per_pixel != -64))
+    ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
 
   if ((!fits_info.simple) || (fits_info.number_axes < 1) ||
       (fits_info.number_axes > 4) || (number_pixels == 0))
@@ -378,7 +426,7 @@ ReadExtension:
     if(!fits_info.extensions_exist)	/* when extensions exists, process further */
       ThrowReaderException(CorruptImageError,ImageTypeNotSupported,image);
 
-    number_pixels = (number_pixels*fits_info.bits_per_pixel) / 8;
+    number_pixels = (number_pixels*AbsoluteValue(fits_info.bits_per_pixel)) / 8;
     number_pixels = ((number_pixels+FITS_BLOCK_SIZE-1) / FITS_BLOCK_SIZE) * FITS_BLOCK_SIZE; /* raw data block size */
     (void) SeekBlob(image,number_pixels,SEEK_CUR);    
   }
@@ -429,8 +477,7 @@ ReadExtension:
     /*
       Initialize image structure.
     */
-    packet_size=fits_info.bits_per_pixel/8;
-    if (packet_size < 0) packet_size = -packet_size;
+    packet_size=AbsoluteValue(fits_info.bits_per_pixel)/8;
         
     number_pixels = image->columns*image->rows;
     if ((number_pixels / image->columns) != image->rows)
@@ -576,6 +623,7 @@ ModuleExport void RegisterFITSImage(void)
   entry->encoder=(EncoderHandler) WriteFITSImage;
   entry->magick=(MagickHandler) IsFITS;
   entry->adjoin=False;
+  entry->seekable_stream=True;
   entry->description="Flexible Image Transport System";
   entry->module="FITS";
   (void) RegisterMagickInfo(entry);
@@ -616,7 +664,7 @@ ModuleExport void UnregisterFITSImage(void)
 */
 int InsertRowHDU(char *buffer, const char *data, int offset)
 {
-  int
+  size_t
     len;
 
   if (data == NULL)
@@ -624,7 +672,8 @@ int InsertRowHDU(char *buffer, const char *data, int offset)
   len = strlen(data);
   len = Min(len,80); /* Each card image is 80 bytes max */
 
-  if (len > FITS_BLOCK_SIZE-offset) len = FITS_BLOCK_SIZE-offset;
+  if (len > (size_t) (FITS_BLOCK_SIZE-offset))
+	len = FITS_BLOCK_SIZE-offset;
 
   (void) strncpy(buffer+offset,data,len);
   return offset +80;
