@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003-2009 GraphicsMagick Group
+% Copyright (C) 2003-2014 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -532,6 +532,7 @@ static char *EscapeParenthesis(const char *text)
   register long
     i;
 
+  /* FIXME: use of a static buffer here makes this function not thread safe! */
   static char
     buffer[MaxTextExtent];
 
@@ -559,6 +560,7 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
 #define ObjectsPerImage  9
 
   char
+    basename[MaxTextExtent],
     buffer[MaxTextExtent],
     date[MaxTextExtent],
     density[MaxTextExtent],
@@ -651,7 +653,7 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
     ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
   (void) memset(xref,0,2048*sizeof(ExtendedSignedIntegralType));
   /*
-    Write Info object.
+    Write Documentation Information Dictionary
   */
   object=0;
   (void) WriteBlobString(image,"%PDF-1.2 \n");
@@ -665,6 +667,10 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
   FormatString(date,"D:%04d%02d%02d%02d%02d%02d",time_meridian->tm_year+1900,
                time_meridian->tm_mon+1,time_meridian->tm_mday,time_meridian->tm_hour,
                time_meridian->tm_min,time_meridian->tm_sec);
+  GetPathComponent(image->filename,BasePath,basename);
+  
+  FormatString(buffer,"/Title (%.1024s)\n",EscapeParenthesis(basename));
+  (void) WriteBlobString(image,buffer);
   FormatString(buffer,"/CreationDate (%.1024s)\n",date);
   (void) WriteBlobString(image,buffer);
   FormatString(buffer,"/ModDate (%.1024s)\n",date);
@@ -811,18 +817,47 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
       geometry.y=(long) text_size;
       FormatString(page_geometry,"%lux%lu",image->columns,image->rows);
       if (image_info->page != (char *) NULL)
-        (void) strlcpy(page_geometry,image_info->page,MaxTextExtent);
+        {
+          (void) strlcpy(page_geometry,image_info->page,MaxTextExtent);
+          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                "Page: %s (from ImageInfo) ", page_geometry);
+
+        }
       else
-        if ((image->page.width != 0) && (image->page.height != 0))
-          (void) FormatString(page_geometry,"%lux%lu%+ld%+ld",image->page.width,
-                              image->page.height,image->page.x,image->page.y);
-        else
-          if (LocaleCompare(image_info->magick,"PDF") == 0)
-            (void) strcpy(page_geometry,PSPageGeometry);
+        {
+          if ((image->page.width != 0) && (image->page.height != 0))
+            {
+              (void) FormatString(page_geometry,"%lux%lu%+ld%+ld",image->page.width,
+                                  image->page.height,image->page.x,image->page.y);
+              (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                    "Page: %s (from Image)", page_geometry);
+            }
+          else
+            {
+              if (LocaleCompare(image_info->magick,"PDF") == 0)
+                {
+                  (void) strlcpy(page_geometry,PSPageGeometry,sizeof(page_geometry));
+                  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                        "Page: %s (defaulted)", page_geometry);
+                }
+              else
+                {
+                  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                        "Page: %s (from Image columns/rows)",
+                                        page_geometry);
+
+                }
+            }
+        }
       (void) GetMagickGeometry(page_geometry,&geometry.x,&geometry.y,
                                &geometry.width,&geometry.height);
       (void) GetGeometry(page_geometry,&media_info.x,&media_info.y,
                          &media_info.width,&media_info.height);
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                            "Image Resolution: %gx%g %s",
+                            image->x_resolution,
+                            image->y_resolution,
+                            ResolutionTypeToString(image->units));
       /*
         Scale relative to dots-per-inch.
       */
@@ -833,12 +868,42 @@ static unsigned int WritePDFImage(const ImageInfo *image_info,Image *image)
       count=GetMagickDimension(density,&x_resolution,&y_resolution,NULL,NULL);
       if (count != 2)
         y_resolution=x_resolution;
-      if (image_info->density != (char *) NULL)
+      /*
+        Use override resolution information if it appears to be valid.
+      */
+      if ((image_info->density != (char *) NULL) &&
+          ((image_info->units == PixelsPerInchResolution) ||
+           (image_info->units == PixelsPerCentimeterResolution)))
         {
-          count=GetMagickDimension(image_info->density,&x_resolution,&y_resolution,NULL,NULL);
+          count=GetMagickDimension(image_info->density,&x_resolution,
+                                   &y_resolution,NULL,NULL);
           if (count != 2)
             y_resolution=x_resolution;
+          if (image_info->units == PixelsPerCentimeterResolution)
+            {
+              x_resolution *= 2.54;
+              y_resolution *= 2.54;
+            }
         }
+      /*
+        Use image resolution information if it appears to be valid.
+      */
+      else if ((image->x_resolution > 0.0) && (image->y_resolution > 0.0) &&
+               ((image->units == PixelsPerInchResolution) ||
+                (image->units == PixelsPerCentimeterResolution)))
+        {
+          x_resolution = image->x_resolution;
+          y_resolution = image->y_resolution;
+          if (image->units == PixelsPerCentimeterResolution)
+            {
+              x_resolution *= 2.54;
+              y_resolution *= 2.54;
+            }
+        }
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                            "PDF Resolution: %gx%g DPI",
+                            x_resolution,y_resolution);
+
       x_scale=(geometry.width*dx_resolution)/x_resolution;
       geometry.width=(unsigned long) (x_scale+0.5);
       y_scale=(geometry.height*dy_resolution)/y_resolution;
