@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 GraphicsMagick Group
+% Copyright (C) 2003-2015 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -21,7 +21,7 @@
 %                                                                             %
 %                              Software Design                                %
 %                              Jaroslav Fojtik                                %
-%                              June 2000 - 2007                               %
+%                              June 2000 - 2015                               %
 %                         Rework for GraphicsMagick                           %
 %                              Bob Friesenhahn                                %
 %                               Feb-May 2003                                  %
@@ -37,14 +37,14 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
-#include "magick/pixel_cache.h"
-#include "magick/color.h"
+#include "magick/colormap.h"
 #include "magick/constitute.h"
 #include "magick/magic.h"
 #include "magick/magick.h"
+#include "magick/pixel_cache.h"
+#include "magick/shear.h"
 #include "magick/tempfile.h"
 #include "magick/transform.h"
-#include "magick/shear.h"
 #include "magick/utility.h"
 
 
@@ -279,50 +279,49 @@ static MagickPassFail InsertRow(unsigned char *p,long y, Image *image, int bpp)
 
     case 2:  /* Convert PseudoColor scanline. */
       {
-        indexes=GetIndexes(image);
-        for (x=0; x < ((long) image->columns-1); x+=2)
+        indexes=AccessMutableIndexes(image);
+        x = 0;
+        while(x < (long)image->columns-3)
           {
-            index=(IndexPacket) ((*p >> 6) & 0x3);
+            index = (IndexPacket)((*p >> 6) & 0x3);
             VerifyColormapIndex(image,index);
-            indexes[x]=index;
+            indexes[x++]=index;
             *q++=image->colormap[index];
-            index=(IndexPacket) ((*p >> 4) & 0x3);
+            index = (IndexPacket)((*p >> 4) & 0x3);
             VerifyColormapIndex(image,index);
-            indexes[x]=index;
+            indexes[x++]=index;
             *q++=image->colormap[index];
-            index=(IndexPacket) ((*p >> 2) & 0x3);
+            index = (IndexPacket)((*p >> 2) & 0x3);
             VerifyColormapIndex(image,index);
-            indexes[x]=index;
-            *q++=image->colormap[index];
-            index=(IndexPacket) ((*p) & 0x3);
+            indexes[x++]=index;
+            *q++ = image->colormap[index];
+            index = (IndexPacket)((*p) & 0x3);
             VerifyColormapIndex(image,index);
-            indexes[x+1]=index;
-            *q++=image->colormap[index];
+            indexes[x++]=index;
+            *q++ = image->colormap[index];
             p++;
-          }
-        if ((image->columns % 4) != 0)
+		  }
+        if(x < (long) image->columns)
           {
-            index=(IndexPacket) ((*p >> 6) & 0x3);
+            index = (IndexPacket) ((*p >> 6) & 0x3);
             VerifyColormapIndex(image,index);
-            indexes[x]=index;
+            indexes[x++] = index;
             *q++=image->colormap[index];
-            if ((image->columns % 4) >= 1)
-
+            if(x < (long) image->columns)
               {
-                index=(IndexPacket) ((*p >> 4) & 0x3);
+                index = (IndexPacket) ((*p >> 4) & 0x3);
                 VerifyColormapIndex(image,index);
-                indexes[x]=index;
+                indexes[x++] = index;
                 *q++=image->colormap[index];
-                if ((image->columns % 4) >= 2)
-
+                if(x < (long) image->columns)
                   {
-                    index=(IndexPacket) ((*p >> 2) & 0x3);
+                    index = (IndexPacket)((*p >> 2) & 0x3);
                     VerifyColormapIndex(image,index);
-                    indexes[x]=index;
+                    indexes[x] = index;
                     *q++=image->colormap[index];
                   }
               }
-            p++;
+            //p++;
           }       
         RetVal = MagickPass;
         break;
@@ -383,12 +382,22 @@ static int UnpackWPGRaster(Image *image,int bpp)
   y=0;
 
   ldblk=(long) ((bpp*image->columns+7)/8);
+  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                        "Raster allocation size: %ld byte%s",
+                        ldblk, (ldblk > 1 ? "s" : ""));
   BImgBuff=MagickAllocateMemory(unsigned char *,(size_t) ldblk);
   if(BImgBuff==NULL) return(-2);
+  (void) memset(BImgBuff,0,(size_t) ldblk);
 
   while(y<(long) image->rows)
     {
-      bbuf=ReadBlobByte(image);
+      i = ReadBlobByte(image);
+	  if(i==EOF)
+	    {
+          MagickFreeMemory(BImgBuff);
+          return(-5);
+        }
+      bbuf = i;
 
       RunCount=bbuf & 0x7F;
       if(bbuf & 0x80)
@@ -646,7 +655,7 @@ unsigned Flags;
 
 
 static Image *ExtractPostscript(Image *image,const ImageInfo *image_info,
-  ExtendedSignedIntegralType PS_Offset,long PS_Size,ExceptionInfo *exception)
+                                ExtendedSignedIntegralType PS_Offset,long PS_Size,ExceptionInfo *exception)
 {
   char
     postscript_file[MaxTextExtent];
@@ -656,15 +665,15 @@ static Image *ExtractPostscript(Image *image,const ImageInfo *image_info,
 
   ImageInfo
     *clone_info;
-    
-  const MagicInfo
-    *magic_info;    
 
   Image
     *image2;
     
   unsigned char
-    magick[2*MaxTextExtent];    
+    magick[2*MaxTextExtent];
+
+  size_t
+    magick_size;
     
 
   if ((clone_info=CloneImageInfo(image_info)) == NULL)
@@ -675,14 +684,14 @@ static Image *ExtractPostscript(Image *image,const ImageInfo *image_info,
   /* Obtain temporary file */
   ps_file=AcquireTemporaryFileStream(postscript_file,BinaryFileIOMode);
   if (!ps_file)
-  {
-    (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Gannot create file stream for PS image");
-    goto FINISH;
-  }
+    {
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Gannot create file stream for PS image");
+      goto FINISH;
+    }
 
   /* Copy postscript to temporary file */
   (void) SeekBlob(image,PS_Offset,SEEK_SET);
-  (void) ReadBlob(image, 2*MaxTextExtent, magick);
+  magick_size=ReadBlob(image, sizeof(magick), magick);
   
   (void) SeekBlob(image,PS_Offset,SEEK_SET);
   while(PS_Size-- > 0)
@@ -691,16 +700,12 @@ static Image *ExtractPostscript(Image *image,const ImageInfo *image_info,
     }
   (void) fclose(ps_file);
   
-    /* Detect file format - Check magic.mgk configuration file. */
-  magic_info=GetMagicInfo(magick,2*MaxTextExtent,exception);
-  if(magic_info == (const MagicInfo *) NULL) goto FINISH_UNL;
-  /*     printf("Detected:%s  \n",magic_info->name); */
-  if(exception->severity != UndefinedException) goto FINISH_UNL;     
-  if(magic_info->name == (char *) NULL) goto FINISH_UNL;
-    
-  (void) strlcpy(clone_info->magick,magic_info->name,MaxTextExtent);
+  /* Detect file format - Check magic.mgk configuration file. */
+  if (GetMagickFileFormat(magick,magick_size,clone_info->magick,
+                           MaxTextExtent,exception) == MagickFail)
+    goto FINISH_UNL;
   
-    /* Read nested image */
+  /* Read nested image */
   /*FormatString(clone_info->filename,"%s:%.1024s",magic_info->name,postscript_file);*/
   FormatString(clone_info->filename,"%.1024s",postscript_file);
   image2=ReadImage(clone_info,exception);
@@ -836,11 +841,13 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
     unsigned int NumOfEntries;
   } WPGColorMapRec;
 
+  /*
   typedef struct {
     unsigned long PS_unknown1;
     unsigned int PS_unknown2;
     unsigned int PS_unknown3;
-  } WPGPSl1Record;  
+  } WPGPSl1Record;
+  */
 
   Image
     *image,
@@ -874,8 +881,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
 
   int
     i,
-    bpp,
-    WPG2Flags;
+    bpp;
 
   int logging;
 
@@ -1049,7 +1055,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                   if (bpp < 24)
                     if ( (image->colors < (1UL<<bpp)) && (bpp != 24) )
                       MagickReallocMemory(PixelPacket *,image->colormap,
-                                          (1<<bpp)*sizeof(PixelPacket));
+                                          (size_t) (1U<<bpp)*sizeof(PixelPacket));
                 }
           
               if (bpp == 1)
@@ -1067,13 +1073,12 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                     }
                 }      
 
-              if(UnpackWPGRaster(image,bpp) < 0)
-                /* The raster cannot be unpacked */
-                {
+			  if(!image_info->ping)
+                if(UnpackWPGRaster(image,bpp) < 0)                
+                {		/* The raster cannot be unpacked */
                 DecompressionFailed:
-                  ThrowReaderException(CoderError,UnableToDecompressImage,
-                                       image)
-                    }
+                  ThrowReaderException(CoderError,UnableToDecompressImage,image)
+                }
 
               if(Rec.RecType==0x14 && BitmapHeader2.RotAngle!=0 && !image_info->ping)
                 {  
@@ -1081,29 +1086,40 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                   if(BitmapHeader2.RotAngle & 0x8000)
                     {
                       rotated_image = FlopImage(image, exception);
-                      rotated_image->blob = image->blob;
-                      image->blob = NULL;
-                      (void) RemoveLastImageFromList(&image);
-                      AppendImageToList(&image,rotated_image);
+                      if (rotated_image != (Image *) NULL)
+                        {
+                          rotated_image->blob = image->blob;
+                          image->blob = NULL;
+                          (void) RemoveLastImageFromList(&image);
+                          AppendImageToList(&image,rotated_image);
+                        }
                     }
                   /* flip command */
                   if(BitmapHeader2.RotAngle & 0x2000)
                     {
                       rotated_image = FlipImage(image, exception);
-                      rotated_image->blob = image->blob;
-                      image->blob = NULL;
-                      (void) RemoveLastImageFromList(&image);
-                      AppendImageToList(&image,rotated_image);		
+                      if (rotated_image != (Image *) NULL)
+                        {
+                          rotated_image->blob = image->blob;
+                          image->blob = NULL;
+                          (void) RemoveLastImageFromList(&image);
+                          AppendImageToList(&image,rotated_image);		
+                        }
                     }
 		
 		  /* rotate command */
                   if(BitmapHeader2.RotAngle & 0x0FFF)
                     {
-                      rotated_image = RotateImage(image, (BitmapHeader2.RotAngle & 0x0FFF), exception);
-                      rotated_image->blob = image->blob;
-                      image->blob = NULL;
-                      (void) RemoveLastImageFromList(&image);
-                      AppendImageToList(&image,rotated_image);		
+                      rotated_image = RotateImage(image,
+                                                  (BitmapHeader2.RotAngle & 0x0FFF),
+                                                  exception);
+                      if (rotated_image != (Image *) NULL)
+                        {
+                          rotated_image->blob = image->blob;
+                          image->blob = NULL;
+                          (void) RemoveLastImageFromList(&image);
+                          AppendImageToList(&image,rotated_image);
+                        }
                     }                
                 }
 
@@ -1216,7 +1232,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
                   if(bpp < 24)
                     if( image->colors<(1UL<<bpp) && bpp!=24 )
                       MagickReallocMemory(PixelPacket *,image->colormap,
-                                          (1<<bpp)*sizeof(PixelPacket));
+                                          (size_t) (1U<<bpp)*sizeof(PixelPacket));
                 }
 
 
@@ -1251,10 +1267,13 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
               if(CTM[0][0]<0 && !image_info->ping)
 		{		/*?? RotAngle=360-RotAngle;*/
 		  rotated_image = FlopImage(image, exception);
-		  rotated_image->blob = image->blob;
-		  image->blob = NULL;
-		  (void) RemoveLastImageFromList(&image);
-		  AppendImageToList(&image,rotated_image);
+                  if (rotated_image != (Image *) NULL)
+                    {
+                      rotated_image->blob = image->blob;
+                      image->blob = NULL;
+                      (void) RemoveLastImageFromList(&image);
+                      AppendImageToList(&image,rotated_image);
+                    }
                   /* Try to change CTM according to Flip - I am not sure, must be checked.		  
                      Tx(0,0)=-1;      Tx(1,0)=0;   Tx(2,0)=0;
                      Tx(0,1)= 0;      Tx(1,1)=1;   Tx(2,1)=0;
@@ -1264,10 +1283,13 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
               if(CTM[1][1]<0 && !image_info->ping)
 		{		/*?? RotAngle=360-RotAngle;*/
 		  rotated_image = FlipImage(image, exception);
-		  rotated_image->blob = image->blob;
-		  image->blob = NULL;
-		  (void) RemoveLastImageFromList(&image);
-		  AppendImageToList(&image,rotated_image);
+                  if (rotated_image != (Image *) NULL)
+                    {
+                      rotated_image->blob = image->blob;
+                      image->blob = NULL;
+                      (void) RemoveLastImageFromList(&image);
+                      AppendImageToList(&image,rotated_image);
+                    }
                   /* Try to change CTM according to Flip - I am not sure, must be checked.
                      float_matrix Tx(3,3);
                      Tx(0,0)= 1;   Tx(1,0)= 0;   Tx(2,0)=0;
@@ -1296,7 +1318,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
               break;
 
 	    case 0x1B:          /*bitmap rectangle*/
-              WPG2Flags = LoadWPG2Flags(image,StartWPG.PosSizePrecision,NULL,&CTM);
+              (void) LoadWPG2Flags(image,StartWPG.PosSizePrecision,NULL,&CTM); /* WPG2Flags */
               break;
             }
         }
@@ -1305,8 +1327,8 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
 
     default:
       {
-        ThrowReaderException(CoderError,DataEncodingSchemeIsNotSupported,image)
-          }
+        ThrowReaderException(CoderError,DataEncodingSchemeIsNotSupported,image);
+      }
     }
 
  Finish:
@@ -1344,6 +1366,8 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
   }
 
   if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),"return");  
+  if(image==NULL)
+    ThrowReaderException(CorruptImageError,ImageFileDoesNotContainAnyImageData,image);
   return(image);   
 }
 
@@ -1378,8 +1402,8 @@ ModuleExport void RegisterWPGImage(void)
   entry=SetMagickInfo("WPG");
   entry->decoder=(DecoderHandler) ReadWPGImage;
   entry->magick=(MagickHandler) IsWPG;
-  entry->description=AcquireString("Word Perfect Graphics");
-  entry->module=AcquireString("WPG");
+  entry->description="Word Perfect Graphics";
+  entry->module="WPG";
   entry->seekable_stream=True;
   (void) RegisterMagickInfo(entry);
 }
